@@ -7,11 +7,14 @@ from typing import NoReturn
 
 from cryspy.A_functions_base.function_2_crystallography_base import \
     calc_cos_ang
-from cryspy.A_functions_base.function_1_matrices import calc_mRmCmRT
+
+from cryspy.A_functions_base.matrix_operations import calc_m1_m2_m1t
+from cryspy.A_functions_base.unit_cell import calc_matrix_t
 
 from cryspy.B_parent_classes.cl_1_item import ItemN
 from cryspy.B_parent_classes.cl_2_loop import LoopN
 from cryspy.B_parent_classes.cl_3_data import DataN
+from cryspy.B_parent_classes.preocedures import take_items_by_class
 
 from cryspy.C_item_loop_classes.cl_1_diffrn_radiation import \
     DiffrnRadiation
@@ -23,7 +26,8 @@ from cryspy.C_item_loop_classes.cl_1_refln_susceptibility import \
 from cryspy.C_item_loop_classes.cl_1_refine_ls import RefineLs
 from cryspy.C_item_loop_classes.cl_1_chi2 import Chi2
 from cryspy.C_item_loop_classes.cl_1_range import Range
-from cryspy.C_item_loop_classes.cl_1_texture import Texture
+from cryspy.C_item_loop_classes.cl_1_setup import Setup
+from cryspy.C_item_loop_classes.cl_1_texture import Texture, TextureL
 from cryspy.C_item_loop_classes.cl_1_exclude import ExcludeL
 
 from cryspy.C_item_loop_classes.cl_1_tof_background import TOFBackground
@@ -62,7 +66,7 @@ class TOF(DataN):
                          TOFMeasL)
     CLASSES_OPTIONAL = (DiffrnRadiation, Chi2, Range,
                         Texture, TOFIntensityIncident, ExcludeL, TOFProcL,
-                        TOFPeakL, RefineLs, ReflnL, ReflnSusceptibilityL)
+                        TOFPeakL, RefineLs, ReflnL, ReflnSusceptibilityL, Setup)
     # CLASSES_INTERNAL = ()
 
     CLASSES = CLASSES_MANDATORY + CLASSES_OPTIONAL
@@ -174,7 +178,7 @@ class TOF(DataN):
                 index_h = peak.numpy_index_h
                 index_k = peak.numpy_index_k
                 index_l = peak.numpy_index_l
-                mult = peak.numpy_index_multiplicity
+                mult = peak.numpy_index_mult
                 cond_2 = True
             except KeyError:
                 if texture is None:
@@ -187,20 +191,20 @@ class TOF(DataN):
                 peak.numpy_index_h = numpy.array(index_h, dtype=int)
                 peak.numpy_index_k = numpy.array(index_k, dtype=int)
                 peak.numpy_index_l = numpy.array(index_l, dtype=int)
-                peak.numpy_index_multiplicity = numpy.array(mult, dtype=int)
+                peak.numpy_index_mult = numpy.array(mult, dtype=int)
                 d_internal_val[f"peak_{crystal.data_name:}"] = peak
                 cond_2 = False
 
             cond_1 = not(crystal.is_variables())
             if cond_1 & cond_2:
-                np_iint_u = peak.numpy_intensity_up
-                np_iint_d = peak.numpy_intensity_down
+                np_iint_u = peak.numpy_intensity_plus
+                np_iint_d = peak.numpy_intensity_minus
             else:
                 np_iint_u, np_iint_d = self.calc_iint(
                     index_h, index_k, index_l, crystal,
                     flag_internal=flag_internal)
-                peak.numpy_intensity_up = np_iint_u
-                peak.numpy_intensity_down = np_iint_d
+                peak.numpy_intensity_plus = np_iint_u
+                peak.numpy_intensity_minus = np_iint_d
 
             cell = crystal.cell
             sthovl_hkl = cell.calc_sthovl(index_h, index_k, index_l)
@@ -271,17 +275,17 @@ class TOF(DataN):
                 peak.numpy_to_items()
 
         tof_proc.numpy_d_spacing = d
-        tof_proc.numpy_intensity_up_net = res_u_1d
-        tof_proc.numpy_intensity_down_net = res_d_1d
+        tof_proc.numpy_intensity_plus_net = res_u_1d
+        tof_proc.numpy_intensity_minus_net = res_d_1d
         tof_proc.numpy_intensity_net = res_u_1d+res_d_1d
         tof_proc.numpy_intensity_diff_total = res_u_1d-res_d_1d
         if flag_polarized:
-            tof_proc.numpy_intensity_up_total = res_u_1d+int_bkgd
-            tof_proc.numpy_intensity_down_total = res_d_1d+int_bkgd
+            tof_proc.numpy_intensity_plus_total = res_u_1d+int_bkgd
+            tof_proc.numpy_intensity_minus_total = res_d_1d+int_bkgd
             tof_proc.numpy_intensity_total = res_u_1d+res_d_1d+int_bkgd+int_bkgd
         else:
-            tof_proc.numpy_intensity_up_total = res_u_1d+0.5*int_bkgd
-            tof_proc.numpy_intensity_down_total = res_d_1d+0.5*int_bkgd
+            tof_proc.numpy_intensity_plus_total = res_u_1d+0.5*int_bkgd
+            tof_proc.numpy_intensity_minus_total = res_d_1d+0.5*int_bkgd
             tof_proc.numpy_intensity_total = res_u_1d+res_d_1d+int_bkgd
 
         if flag_internal:
@@ -330,10 +334,10 @@ class TOF(DataN):
 
         np_time = tof_meas.numpy_time
         if flag_polarized:
-            int_u_exp = tof_meas.numpy_intensity_up
-            sint_u_exp = tof_meas.numpy_intensity_up_sigma
-            int_d_exp = tof_meas.numpy_intensity_down
-            sint_d_exp = tof_meas.numpy_intensity_down_sigma
+            int_u_exp = tof_meas.numpy_intensity_plus
+            sint_u_exp = tof_meas.numpy_intensity_plus_sigma
+            int_d_exp = tof_meas.numpy_intensity_minus
+            sint_d_exp = tof_meas.numpy_intensity_minus_sigma
         else:
             int_exp = tof_meas.numpy_intensity
             sint_exp = tof_meas.numpy_intensity_sigma
@@ -364,10 +368,10 @@ class TOF(DataN):
             flag_polarized=flag_polarized)
 
         if flag_polarized:
-            tof_proc.numpy_intensity_up = int_u_exp_in
-            tof_proc.numpy_intensity_up_sigma = sint_u_exp_in
-            tof_proc.numpy_intensity_down = int_d_exp_in
-            tof_proc.numpy_intensity_down_sigma = sint_d_exp_in
+            tof_proc.numpy_intensity_plus = int_u_exp_in
+            tof_proc.numpy_intensity_plus_sigma = sint_u_exp_in
+            tof_proc.numpy_intensity_minus = int_d_exp_in
+            tof_proc.numpy_intensity_minus_sigma = sint_d_exp_in
             tof_proc.numpy_intensity = int_u_exp_in+int_d_exp_in
             tof_proc.numpy_intensity_sigma = numpy.sqrt(
                 numpy.square(sint_u_exp_in) + numpy.square(sint_d_exp_in))
@@ -375,8 +379,8 @@ class TOF(DataN):
             tof_proc.numpy_intensity = int_exp_in
             tof_proc.numpy_intensity_sigma = sint_exp_in
 
-        int_u_mod = tof_proc.numpy_intensity_up_total
-        int_d_mod = tof_proc.numpy_intensity_down_total
+        int_u_mod = tof_proc.numpy_intensity_plus_total
+        int_d_mod = tof_proc.numpy_intensity_minus_total
 
         if flag_polarized:
             sint_sum_exp_in = (sint_u_exp_in**2 + sint_d_exp_in**2)**0.5
@@ -489,6 +493,8 @@ class TOF(DataN):
             - refln_s: ReflnSusceptibilityL object of cryspy library
               (nuclear structure factor)
         """
+        index_hkl = numpy.stack([index_h, index_k, index_l], axis=0)
+
         if flag_internal:
             try:
                 d_internal_val = self.d_internal_val
@@ -546,33 +552,25 @@ class TOF(DataN):
             sft_32 = refln_s.numpy_chi_32_calc
             sft_33 = refln_s.numpy_chi_33_calc
 
-            sftm_11 = refln_s.numpy_moment_11_calc
-            sftm_12 = refln_s.numpy_moment_12_calc
-            sftm_13 = refln_s.numpy_moment_13_calc
-            sftm_21 = refln_s.numpy_moment_21_calc
-            sftm_22 = refln_s.numpy_moment_22_calc
-            sftm_23 = refln_s.numpy_moment_23_calc
-            sftm_31 = refln_s.numpy_moment_31_calc
-            sftm_32 = refln_s.numpy_moment_32_calc
-            sftm_33 = refln_s.numpy_moment_33_calc
 
-            _11, _12 = sftm_11+field*sft_11, sftm_12+field*sft_12
-            _21, _13 = sftm_21+field*sft_21, sftm_13+field*sft_13
-            _22, _23 = sftm_22+field*sft_22, sftm_23+field*sft_23
-            _31, _32 = sftm_31+field*sft_31, sftm_32+field*sft_32
-            _33 = sftm_33+field*sft_33
-            _ij = (_11, _12, _13, _21, _22, _23, _31, _32, _33)
+            _11, _12 = field*sft_11, field*sft_12
+            _21, _13 = field*sft_21, field*sft_13
+            _22, _23 = field*sft_22, field*sft_23
+            _31, _32 = field*sft_31, field*sft_32
+            _33 = field*sft_33
+            _ij = numpy.stack([_11, _12, _13, _21, _22, _23, _31, _32, _33], axis=0)
             cell = crystal.cell
 
-            # k_loc = cell.calc_k_loc(h, k, l)
-            t_ij = cell.calc_m_t(index_h, index_k, index_l)
-            # FIXME: I would like to recheck the expression for T
-            #        and expression SIGMA = T^T CHI T
-            t_tr_ij = (t_ij[0], t_ij[3], t_ij[6],
-                       t_ij[1], t_ij[4], t_ij[7],
-                       t_ij[2], t_ij[5], t_ij[8])
-            th_11, th_12, th_13, th_21, th_22, th_23, th_31, th_32, th_33 = \
-                calc_mRmCmRT(t_tr_ij, _ij)
+            unit_cell_parameters = numpy.array([
+                cell.length_a, cell.length_b, cell.length_c,
+                cell.angle_alpha*numpy.pi/180.,
+                cell.angle_beta*numpy.pi/180.,
+                cell.angle_gamma*numpy.pi/180.], dtype=float)
+            
+            t_ij = calc_matrix_t(index_hkl, unit_cell_parameters, flag_unit_cell_parameters=False)[0]
+            #SIGMA = T CHI T^-1 = T chi T^T (because T is rotation matrix, therefore T^-1 = T^T)
+            th_ij = calc_m1_m2_m1t(t_ij, _ij)[0]
+            th_11, th_12, th_22 = th_ij[0], th_ij[1], th_ij[4]
 
             # fm_p_sq = (field**2)*abs(0.5*(th_11*th_11.conjugate()+
             #            th_22*th_22.conjugate())+th_12*th_12.conjugate())
@@ -687,92 +685,329 @@ class TOF(DataN):
     def plots(self):
         if self.is_attribute("tof_proc"):
             tof_proc = self.tof_proc
-            fig_s, ax_s = tof_proc.plot_sum()
-            fig_d_ax_d = tof_proc.plot_diff()
-            flag_d = fig_d_ax_d is not None
-            if flag_d:
-                fig_d, ax_d = fig_d_ax_d
-                y_min_d, y_max_d = ax_d.get_ylim()
-                y_dist_d = y_max_d-y_min_d
-                y_step_d = 0.
-
-            y_min_s, y_max_s = ax_s.get_ylim()
-            y_dist_s = y_max_s-y_min_s
-            y_step_s = 0.
-            for item in self.items:
-                if isinstance(item, TOFPeakL):
-                    np_tth = item.numpy_time
-                    ax_s.plot(np_tth, 0.*np_tth+y_min_s-y_step_s, "|", label=item.loop_name)
-                    y_step_s += 0.05*y_dist_s
-                    if flag_d:
-                        ax_d.plot(np_tth, 0.*np_tth+y_min_d-y_step_d, "|", label=item.loop_name)
-                        y_step_d += 0.05*y_dist_d
-            ax_s.set_title(self.data_name)
-            ax_s.legend(loc='upper right')
-            if flag_d:
-                ax_d.set_title(self.data_name)
-                ax_d.legend(loc='upper right')
-            if flag_d:
-                res = [(fig_s, ax_s), (fig_d, ax_d)]
-            else:
-                res = [(fig_s, ax_s)]
-            return res
-            
-            return self.tof_proc.plots()
+            res = tof_proc.plot_sum_diff()
+            return [res,]
         elif self.is_attribute("tof_meas"):
             return self.tof_meas.plots()
 
-# s_cont = """
-#   data_tof
+    def get_dictionary(self):
+        self.form_object()
+        dict_tof = {}
 
-# _tof_profile_sigma2 0.00000
-# _tof_profile_sigma1 61.67600
-# _tof_profile_sigma0 6.19600
-# _tof_profile_gamma2 0.00000
-# _tof_profile_gamma1 0.60400
-# _tof_profile_gamma0 0.00000
+        phase, range_, tof_background = None, None, None
+        exclude = None
+        tof_meas, tof_parameters, tof_profile = None, None, None
+        diffrn_radiation, texture = None, None
+        setup = None
 
-# _range_time_min 3001.589
-# _range_time_max 19000.0
+        l_obj = take_items_by_class(self, (PhaseL, ))
+        if len(l_obj) > 0:
+            phase = l_obj[0]
 
-# _tof_background_time_max 19000.0
-# _tof_background_coeff1 24832.850
-# _tof_background_coeff2 6139.244
-# _tof_background_coeff3 8063.472
-# _tof_background_coeff4 3125.050
-# _tof_background_coeff5 2566.956
-# _tof_background_coeff6 311.077
-# _tof_background_coeff7 837.348
-# _tof_background_coeff8 -103.742
-# _tof_background_coeff9 -11.806
+        l_obj = take_items_by_class(self, (Range, ))
+        if len(l_obj) > 0:
+            range_ = l_obj[0]
 
-# loop_
-# _phase_label
-# _phase_scale
-# _phase_igsize
-# cecual   0.024678(78)   0.0  
+        l_obj = take_items_by_class(self, (TOFBackground, ))
+        if len(l_obj) > 0:
+            tof_background = l_obj[0]
 
-# loop_
-# _tof_meas_time
-# _tof_meas_intensity
-# _tof_meas_intensity_sigma
-# 3001.589    40409    00462  
-# 3003.090    40171    00460  
+        l_obj = take_items_by_class(self, (ExcludeL, ))
+        if len(l_obj) > 0:
+            exclude = l_obj[0]
 
-# _tof_parameters_zero 2.921
-# _tof_parameters_dtt1 6167.247
-# _tof_parameters_dtt1 -2.280
-# _tof_parameters_2theta_bank 145.
+        l_obj = take_items_by_class(self, (TOFMeasL, ))
+        if len(l_obj) > 0:
+            tof_meas = l_obj[0]
+
+        l_obj = take_items_by_class(self, (TOFParameters, ))
+        if len(l_obj) > 0:
+            tof_parameters = l_obj[0]
+
+        l_obj = take_items_by_class(self, (TOFProfile, ))
+        if len(l_obj) > 0:
+            tof_profile = l_obj[0]
+
+        l_obj = take_items_by_class(self, (DiffrnRadiation, ))
+        if len(l_obj) > 0:
+            diffrn_radiation = l_obj[0]
+
+        l_obj = take_items_by_class(self, (TextureL, ))
+        if len(l_obj) > 0:
+            texture = l_obj[0]
+
+        l_obj = take_items_by_class(self, (Setup, ))
+        if len(l_obj) > 0:
+            setup = l_obj[0]
+
+        dict_tof["name"] = self.data_name
+        dict_tof["type_name"] = self.get_name()
+
+        if phase is not None:
+            dict_tof["phase_name"] = numpy.array(phase.label, dtype=str)
+
+            if phase.is_attribute("igsize"):
+                dict_tof["phase_ig"] = numpy.array(phase.igsize, dtype=float)
+                dict_tof["flags_phase_ig"] = numpy.array(phase.igsize_refinement, dtype=bool)
+            else:
+                dict_tof["phase_ig"] = numpy.zeros((len(phase.items),), dtype=float)
+                dict_tof["flags_phase_ig"] = numpy.zeros((len(phase.items),), dtype=bool)
+
+            if phase.is_attribute("scale"):
+                dict_tof["phase_scale"] = numpy.array(phase.scale, dtype=float)
+                dict_tof["flags_phase_scale"] = numpy.array(phase.scale_refinement, dtype=bool)
+            else:
+                dict_tof["phase_scale"] = numpy.zeros((len(phase.items),), dtype=float)
+                dict_tof["flags_phase_scale"] = numpy.zeros((len(phase.items),), dtype=bool)
+
+        if tof_background is not None:
+            dict_tof["background_time_max"] = tof_background.time_max
+            dict_tof["background_coefficients"] = tof_background.get_coefficients()
+            dict_tof["flags_background_coefficients"] = tof_background.get_flags_coefficients()
+            
+
+        if tof_meas is not None:
+            time = numpy.array(tof_meas.time, dtype=float)
+
+            time_min = range_.time_min
+            time_max = range_.time_max
+            dict_tof["time_min"] = time_min
+            dict_tof["time_max"] = time_max
 
 
-# loop_
-# _exclude_time_low
-# _exclude_time_high
-# 2000.1    3000.0   
-# 19000.0   20000.0  
+            flag_in = numpy.logical_and(
+                time >= time_min,
+                time <= time_max)
+            
+            dict_tof["time"] = time[flag_in]
+            if tof_meas.is_attribute("intensity_plus"):
+                int_plus = numpy.array(tof_meas.intensity_plus, dtype=float)[flag_in]
+                s_int_plus = numpy.array(tof_meas.intensity_plus_sigma, dtype=float)[flag_in]
+                int_minus = numpy.array(tof_meas.intensity_minus, dtype=float)[flag_in]
+                s_int_minus = numpy.array(tof_meas.intensity_minus_sigma, dtype=float)[flag_in]
+                dict_tof["signal_exp_plus"] = numpy.stack([int_plus, s_int_plus], axis=0)
+                dict_tof["signal_exp_minus"] = numpy.stack([int_minus, s_int_minus], axis=0)
+            else:
+                int_sum = numpy.array(tof_meas.intensity, dtype=float)[flag_in]
+                s_int_sum = numpy.array(tof_meas.intensity_sigma, dtype=float)[flag_in]
+                dict_tof["signal_exp"] = numpy.stack([int_sum, s_int_sum], axis=0)
+
+            time_in_range = time[flag_in]
+            flag_exclude = numpy.zeros(time_in_range.shape, dtype=bool)
+            if exclude is not None:
+                for item_e in exclude.items:
+                    flag_in_1 = numpy.logical_and(
+                        time_in_range >= item_e.time_min,
+                        time_in_range <= item_e.time_max)
+                    flag_exclude = numpy.logical_or(flag_exclude, flag_in_1)
+            dict_tof["excluded_points"] = flag_exclude
+
+
+        if tof_parameters is not None:
+            dict_tof["ttheta_bank"] = tof_parameters.ttheta_bank * numpy.pi/180
+            dict_tof["neutron_type"] = tof_parameters.neutrons
+            dict_tof["zero"] = tof_parameters.zero
+            dict_tof["dtt1"] = tof_parameters.dtt1
+            if tof_parameters.is_attribute("dtt2"):
+                dict_tof["dtt2"] = tof_parameters.dtt2
+            if tof_parameters.is_attribute("zerot"):
+                dict_tof["zerot"] = tof_parameters.zerot
+            if tof_parameters.is_attribute("dtt1t"):
+                dict_tof["dtt1t"] = tof_parameters.dtt1t
+            if tof_parameters.is_attribute("dtt2t"):
+                dict_tof["dtt2t"] = tof_parameters.dtt2t
+
+
+        if tof_profile is not None:
+            l_alpha = []
+            l_alpha_refinement = []
+            for numb in range(2):
+                if tof_profile.is_attribute(f"alpha{numb:}"):
+                    l_alpha.append(getattr(tof_profile, f"alpha{numb:}"))
+                    l_alpha_refinement.append(getattr(tof_profile, f"alpha{numb:}_refinement"))
+                else:
+                    break
+
+            l_beta = []
+            l_beta_refinement = []
+            for numb in range(2):
+                if tof_profile.is_attribute(f"beta{numb:}"):
+                    l_beta.append(getattr(tof_profile, f"beta{numb:}"))
+                    l_beta_refinement.append(getattr(tof_profile, f"beta{numb:}_refinement"))
+                else:
+                    break
+
+            l_gamma = []
+            l_gamma_refinement = []
+            for numb in range(2):
+                if tof_profile.is_attribute(f"gamma{numb:}"):
+                    l_gamma.append(getattr(tof_profile, f"gamma{numb:}"))
+                    l_gamma_refinement.append(getattr(tof_profile, f"gamma{numb:}_refinement"))
+                else:
+                    break
+
+            l_sigma = []
+            l_sigma_refinement = []
+            for numb in range(3):
+                if tof_profile.is_attribute(f"sigma{numb:}"):
+                    l_sigma.append(getattr(tof_profile, f"sigma{numb:}"))
+                    l_sigma_refinement.append(getattr(tof_profile, f"sigma{numb:}_refinement"))
+                else:
+                    break
+
+            if tof_profile.is_attribute("peak_shape"):
+                peak_shape = tof_profile.peak_shape
+            else:
+                peak_shape = "pseudo-Voigt"
+
+            dict_tof["profile_alphas"] = numpy.array(l_alpha, dtype=float)
+            dict_tof["flags_profile_alphas"] = numpy.array(l_alpha_refinement, dtype=float)
+            dict_tof["profile_betas"] = numpy.array(l_beta, dtype=float)
+            dict_tof["flags_profile_betas"] = numpy.array(l_beta_refinement, dtype=float)
+            dict_tof["profile_gammas"] = numpy.array(l_gamma, dtype=float)
+            dict_tof["flags_profile_gammas"] = numpy.array(l_gamma_refinement, dtype=float)
+            dict_tof["profile_sigmas"] = numpy.array(l_sigma, dtype=float)
+            dict_tof["flags_profile_sigmas"] = numpy.array(l_sigma_refinement, dtype=float)
+            dict_tof["profile_peak_shape"] = peak_shape
+
+        if diffrn_radiation is not None:
+            beam_polarization = diffrn_radiation.polarization
+            flipper_efficiency = diffrn_radiation.efficiency
+            dict_tof["beam_polarization"] = numpy.array([beam_polarization], dtype=float)
+            dict_tof["flipper_efficiency"] = numpy.array([flipper_efficiency], dtype=float)
+            dict_tof["flags_beam_polarization"] = numpy.array([diffrn_radiation.polarization_refinement], dtype=bool)
+            dict_tof["flags_flipper_efficiency"] = numpy.array([diffrn_radiation.efficiency_refinement], dtype=bool)
+
+
+        if texture is not None:
+            dict_tof["texture_name"] = numpy.array(texture.label, dtype=str)
+            dict_tof["texture_g1"] = numpy.array(texture.g_1, dtype=float)
+            dict_tof["texture_g2"] = numpy.array(texture.g_2, dtype=float)
+            dict_tof["texture_axis"] = numpy.array(
+                [texture.h_ax, texture.k_ax, texture.l_ax], dtype=float)
+            dict_tof["flags_texture_g1"] = numpy.array(texture.g_1_refinement, dtype=bool)
+            dict_tof["flags_texture_g2"] = numpy.array(texture.g_2_refinement, dtype=bool)
+            dict_tof["flags_texture_axis"] = numpy.array(
+                [texture.h_ax_refinement, 
+                 texture.k_ax_refinement, 
+                 texture.l_ax_refinement], dtype=bool)
+
+
+        if setup is not None:
+            dict_tof["magnetic_field"] = numpy.array([setup.field], dtype=float)
+
+        return dict_tof
+
+
+    def take_parameters_from_dictionary(self, ddict_diffrn, l_parameter_name: list=None, l_sigma: list=None):
+        keys = ddict_diffrn.keys()
+        if l_parameter_name is not None:
+            parameter_label = [hh[0] for hh in l_parameter_name]
+        else:
+            parameter_label = []
+        if "beam_polarization" in keys:
+            self.diffrn_radiation.polarization = ddict_diffrn["beam_polarization"]
+
+        if "flipper_efficiency" in keys:
+            self.diffrn_radiation.efficiency = ddict_diffrn["flipper_efficiency"]
+
+        if "phase_scale" in keys:
+            hh = ddict_diffrn["phase_scale"]
+            for i_item, item in enumerate(self.phase.items):
+                item.scale = float(hh[i_item])
+
+        if "phase_ig" in keys:
+            hh = ddict_diffrn["phase_ig"]
+            for i_item, item in enumerate(self.phase.items):
+                item.igsize = float(hh[i_item])
+
+
+        if len(parameter_label) > 0:
+            for name, sigma in zip(l_parameter_name, l_sigma):
+                if name[0] == "phase_scale":
+                    self.phase.items[name[1][0]].scale_sigma = float(sigma)
+                if name[0] == "phase_ig":
+                    self.phase.items[name[1][0]].igsize_sigma = float(sigma)
+                if name[0] == "beam_polarization":
+                    self.diffrn_radiation.polarization_sigma = float(sigma)
+                if name[0] == "flipper_efficiency":
+                    self.diffrn_radiation.efficiency_sigma = float(sigma)
+                if name[0] == "texture_g1":
+                    self.texture.items[name[1][0]].g1_sigma = float(sigma)
+                if name[0] == "texture_g2":
+                    self.texture.items[name[1][0]].g2_sigma = float(sigma)
+                if name[0] == "texture_axis":
+                    ind_param, ind_a = name[1]
+                    if ind_param == 0:
+                        self.texture.items[ind_a].h_ax_sigma = float(sigma)
+                    if ind_param == 1:
+                        self.texture.items[ind_a].k_ax_sigma = float(sigma)
+                    if ind_param == 2:
+                        self.texture.items[ind_a].l_ax_sigma = float(sigma)
+
+        if (("signal_plus" in keys) and ("signal_minus" in keys)):
+            tof_proc = TOFProcL()
+            tof_proc.numpy_time = numpy.round(ddict_diffrn["time"], decimals=5)
+            tof_proc.numpy_intensity_plus_net = numpy.round(ddict_diffrn["signal_plus"], decimals=5)
+            tof_proc.numpy_intensity_minus_net = numpy.round(ddict_diffrn["signal_minus"], decimals=5)
+            if "signal_exp_plus" in keys:
+                tof_proc.numpy_intensity_plus = numpy.round(ddict_diffrn["signal_exp_plus"][0], decimals=5)
+                tof_proc.numpy_intensity_plus_sigma = numpy.round(ddict_diffrn["signal_exp_plus"][1], decimals=5)
+                tof_proc.numpy_intensity_minus = numpy.round(ddict_diffrn["signal_exp_minus"][0], decimals=5)
+                tof_proc.numpy_intensity_minus_sigma = numpy.round(ddict_diffrn["signal_exp_minus"][1], decimals=5)
+            else:
+                tof_proc.numpy_intensity = numpy.round(ddict_diffrn["signal_exp"][0], decimals=5)
+                tof_proc.numpy_intensity_sigma = numpy.round(ddict_diffrn["signal_exp"][1], decimals=5)
+            tof_proc.numpy_intensity_bkg_calc = numpy.round(ddict_diffrn["signal_background"], decimals=5)
+            tof_proc.numpy_excluded = ddict_diffrn["excluded_points"]
+            tof_proc.numpy_to_items()
+            self.tof_proc = tof_proc
+
+        l_tof_peak = []
+        l_refln = []
+        for item_phase in self.phase.items:
+            s_label = f"dict_in_out_{item_phase.label:}"
+            if s_label in keys:
+                dict_crystal = ddict_diffrn[s_label]
+                dict_crystal_keys = dict_crystal.keys()
+                if (("index_hkl" in dict_crystal_keys) and  ("time_hkl" in dict_crystal_keys)):
+                    index_hkl = dict_crystal["index_hkl"]
+                    time_hkl = dict_crystal["time_hkl"]
+
+                    tof_peak = TOFPeakL(loop_name = item_phase.label)
+                    int_plus_max, int_minus_max = None, None
+                    if "iint_plus_with_factors" in dict_crystal_keys:
+                        int_plus_max = dict_crystal["iint_plus_with_factors"]
+                    if "iint_minus_with_factors" in dict_crystal_keys:
+                        int_minus_max = dict_crystal["iint_minus_with_factors"]
+
+                    if "f_nucl":
+                        refln = ReflnL(loop_name = item_phase.label)
+                        refln.numpy_index_h = index_hkl[0]
+                        refln.numpy_index_k = index_hkl[1]
+                        refln.numpy_index_l = index_hkl[2]
+                        refln.numpy_a_calc = dict_crystal["f_nucl"].real
+                        refln.numpy_b_calc = dict_crystal["f_nucl"].imag
+                        refln.numpy_to_items()
+                        l_refln.append(refln)
+
+                    if not((int_plus_max is None) and (int_minus_max is None)):
+                        tof_peak.numpy_index_h = index_hkl[0]
+                        tof_peak.numpy_index_k = index_hkl[1]
+                        tof_peak.numpy_index_l = index_hkl[2]
+                        tof_peak.numpy_time = numpy.round(time_hkl, decimals=3)
+                        tof_peak.numpy_intensity_plus = int_plus_max
+                        tof_peak.numpy_intensity_minus = int_minus_max
+                        tof_peak.numpy_to_items()
+                        l_tof_peak.append(tof_peak)
+
+        if len(l_tof_peak) > 0:
+            self.add_items(l_tof_peak)
+        if len(l_refln) > 0:
+            self.add_items(l_refln)
+
+    def estimate_background(self):
+        tof_background = self.tof_background
+        tof_proc = self.tof_proc
+        tof_proc.estimate_background(tof_background)
 
-# """
-# obj = TOF.from_cif(s_cont)
-# print(obj)
-# for var_name in obj.get_variable_names():
-#     print(var_name)
